/*********************************************************************
* Software License Agreement (BSD License)
*
*  Copyright (c) 2010, Rice University
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without
*  modification, are permitted provided that the following conditions
*  are met:
*
*   * Redistributions of source code must retain the above copyright
*     notice, this list of conditions and the following disclaimer.
*   * Redistributions in binary form must reproduce the above
*     copyright notice, this list of conditions and the following
*     disclaimer in the documentation and/or other materials provided
*     with the distribution.
*   * Neither the name of the Rice University nor the names of its
*     contributors may be used to endorse or promote products derived
*     from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
*  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
*  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
*  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
*  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
*  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
*  POSSIBILITY OF SUCH DAMAGE.
*********************************************************************/

/* Author: Caleb Voss */

#ifndef OMPL_EXTENSION_MORSE_GOAL_
#define OMPL_EXTENSION_MORSE_GOAL_

#include "ompl/base/Goal.h"
#include "ompl/base/SpaceInformation.h"

#include <limits>

namespace ompl
{
    namespace base
    {
        /** \brief This is a goal class that is more amenable to Python  */
        class MorseGoal : public Goal
        {
        public:
            MorseGoal(SpaceInformationPtr si) : Goal(si), distance_(std::numeric_limits<double>::max())
            {
            }

            /** \brief Where MORSE will store the distance to goal during an isSatisfied() call */
            mutable double distance_;

            /** \brief Return true if \e state satisfies the goal */
            bool isSatisfied(const State *state) const override;

            /** \brief Return true if \e state satisfies the goal, and store the distance
                to the goal in \e distance */
            bool isSatisfied(const State *state, double *distance) const override;

            /** \brief To be implemented in Python; behaves like isSatisfied(state, &distance_) */
            virtual bool isSatisfied_Py(const State *state) const = 0;
        };
    }
}

#endif
